package ppxlib

  1. Overview
  2. Docs
Legend:
Page
Library
Module
Module type
Parameter
Class
Class type
Module Astlib.Migrate_504_503Source

Sourcemodule From = Ast_504
Sourcemodule To = Ast_503
Sourcemodule Bivariant_param : sig ... end
Sourceval bivariant_error : loc:Location.t -> 'a
Sourceval copy_class_infos_ : 'f0 'g0. ('f0 -> 'g0) -> 'f0 Ast_504.Parsetree.class_infos -> bool * 'g0 Ast_503.Parsetree.class_infos
Sourceval copy_include_infos : 'f0 'g0. ('f0 -> 'g0) -> 'f0 Ast_504.Parsetree.include_infos -> 'g0 Ast_503.Parsetree.include_infos
Sourceval copy_open_infos : 'f0 'g0. ('f0 -> 'g0) -> 'f0 Ast_504.Parsetree.open_infos -> 'g0 Ast_503.Parsetree.open_infos
Sourceval copy_Longident_t : Ast_504.Longident.t -> Longident.t
Sourceval copy_loc : 'f0 'g0. ('f0 -> 'g0) -> 'f0 Ast_504.Asttypes.loc -> 'g0 Ast_503.Asttypes.loc
Sourceval copy_location : Location.t -> Location.t

The functions below are provided to keep a coherent and stable API with other migrate_X_Y modules while allowing us to locally define variations of those functions used to encode 5.4 features into the 5.3 AST.

Sourceval copy_class_infos : 'f0 'g0. ('f0 -> 'g0) -> 'f0 Ast_504.Parsetree.class_infos -> 'g0 Ast_503.Parsetree.class_infos