package ppxlib

  1. Overview
  2. Docs
Legend:
Page
Library
Module
Module type
Parameter
Class
Class type
Module Astlib.Migrate_505_504Source

Sourcemodule From = Ast_505
Sourcemodule To = Ast_504
Sourcemodule External_type : sig ... end
Sourceval copy_location : 'a -> 'a
Sourceval copy_loc : 'f0 'g0. ('f0 -> 'g0) -> 'f0 Ast_505.Asttypes.loc -> 'g0 Ast_504.Asttypes.loc
Sourceval copy_class_infos : 'f0 'g0. ('f0 -> 'g0) -> 'f0 Ast_505.Parsetree.class_infos -> 'g0 Ast_504.Parsetree.class_infos
Sourceval copy_open_infos : 'f0 'g0. ('f0 -> 'g0) -> 'f0 Ast_505.Parsetree.open_infos -> 'g0 Ast_504.Parsetree.open_infos
Sourceval copy_include_infos : 'f0 'g0. ('f0 -> 'g0) -> 'f0 Ast_505.Parsetree.include_infos -> 'g0 Ast_504.Parsetree.include_infos

The functions below are provided to keep a coherent and stable API with other migrate_X_Y modules while allowing us to locally define variations of those functions used to encode 5.5 features into the 5.4 AST.